<?php

namespace FiloBlu\ExtMageWorxSeoRedirects\Model\Import;

use Magento\ImportExport\Model\Import\ErrorProcessing\ProcessingErrorAggregatorInterface;

class Redirects extends \Magento\ImportExport\Model\Import\Entity\AbstractEntity {

    const REDIRECT_CODE = 'redirect_code';
    const REQUEST_ENTITY_TYPE = 'request_entity_type';
    const REQUEST_ENTITY_IDENTIFIER = 'request_entity_identifier';
    const TARGET_ENTITY_TYPE = 'target_entity_type';
    const TARGET_ENTITY_IDENTIFIER = 'target_entity_identifier';
    const STORE_ID = 'store_id';

    protected $logInHistory = true;
    protected $needColumnCheck = false;
    protected $customRedirectFactory;
    
//    protected $_modelRedirect;
    protected $_permanentAttributes = [
        self::REDIRECT_CODE,
        self::REQUEST_ENTITY_TYPE,
        self::REQUEST_ENTITY_IDENTIFIER,
        self::TARGET_ENTITY_TYPE,
        self::TARGET_ENTITY_IDENTIFIER,
        self::STORE_ID,
    ];
    protected $validColumnNames = [
        self::REDIRECT_CODE,
        self::REQUEST_ENTITY_TYPE,
        self::REQUEST_ENTITY_IDENTIFIER,
        self::TARGET_ENTITY_TYPE,
        self::TARGET_ENTITY_IDENTIFIER,
        self::STORE_ID,
    ];

    public function __construct(
    \Magento\Framework\Json\Helper\Data $jsonHelper, \Magento\ImportExport\Helper\Data $importExportData, \Magento\ImportExport\Model\ResourceModel\Import\Data $importData, \Magento\Framework\App\ResourceConnection $resource, \Magento\ImportExport\Model\ResourceModel\Helper $resourceHelper, ProcessingErrorAggregatorInterface $errorAggregator, \MageWorx\SeoRedirects\Model\Redirect\CustomRedirectFactory $customRedirectFactory
    ) {
        $this->jsonHelper = $jsonHelper;
        $this->_importExportData = $importExportData;
        $this->_resourceHelper = $resourceHelper;
        $this->_dataSourceModel = $importData;
        $this->_resource = $resource;
        $this->_connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->errorAggregator = $errorAggregator;
        $this->redirectFactory = $customRedirectFactory;
    }

    public function getValidColumnNames() {
        return $this->validColumnNames;
    }

    /**
     * Create Advanced price data from raw data.
     *
     * @throws \Exception
     * @return bool Result of operation.
     */
    protected function _importData() {

        if (\Magento\ImportExport\Model\Import::BEHAVIOR_APPEND == $this->getBehavior()) {
            $this->updateData();
        }

        return true;
    }

    /**
     * Entity type code getter.
     *
     * @return string
     */
    public function getEntityTypeCode() {
        return 'filobluseoredirects_import';
    }

    public function validateRow(array $rowData, $rowNum) {
        return !$this->getErrorAggregator()->isRowInvalid($rowNum);
    }

    protected function updateData() {
        $behavior = $this->getBehavior();
        while ($bunch = $this->_dataSourceModel->getNextBunch()) {
            foreach ($bunch as $rowNum => $rowData) {
//                if (!$this->validateRow($rowData, $rowNum)) {
//                    $this->addRowError(ERROR_INVALID_DATA, $rowNum);
//                    continue;
//                }
//                if ($this->getErrorAggregator()->hasToBeTerminated()) {
//                    $this->getErrorAggregator()->addRowToSkip($rowNum);
//                    continue;
//                }
                if (\Magento\ImportExport\Model\Import::BEHAVIOR_APPEND == $behavior) {
                    $modelRedirect = $this->redirectFactory->create();
                    $modelRedirect->addData($rowData);
                    $modelRedirect->setIsAutogenerated(0);
                    $modelRedirect->setStatus(1);
                    $modelRedirect->save();
                }
            }
        }

        return $this;
    }

}
